<?php

namespace Bixo\Company\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Company\Http\Requests\CompaniesRequest;
use Bixo\Company\Interfaces\CompaniesRepositoryInterface;
use Bixo\Company\Models\Companies;
use Bixo\Company\Forms\Companies as Form;

/**
 * APIController  class for companies.
 */
class CompaniesAPIController extends BaseController
{

    /**
     * Initialize companies resource controller.
     *
     * @param type CompaniesRepositoryInterface $companies
     *
     * @return null
     */
    public function __construct(CompaniesRepositoryInterface $companies)
    {
        parent::__construct();
        $this->repository = $companies;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Company\Repositories\Criteria\CompaniesResourceCriteria::class);
    }

    /**
     * Display a list of companies.
     *
     * @return Response
     */
    public function index(CompaniesRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Company\Repositories\Presenter\CompaniesPresenter::class)
            ->paginate();
    }

    /**
     * Display companies.
     *
     * @param Request $request
     * @param Model   $companies
     *
     * @return Response
     */
    public function show(CompaniesRequest $request, Companies $companies)
    {
        return $companies->setPresenter(\Bixo\Company\Repositories\Presenter\CompaniesListPresenter::class);
        ;
    }

    /**
     * Create new companies.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CompaniesRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('company::companies.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('company/companies/' . $companies->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('company/companies');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the companies.
     *
     * @param Request $request
     * @param Model   $companies
     *
     * @return Response
     */
    public function update(CompaniesRequest $request, Companies $companies)
    {
        try {
            $data = $request->all();

            $companies->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('company::companies.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('company/companies/' . $companies->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('company/companies/' . $companies->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the companies.
     *
     * @param Model   $companies
     *
     * @return Response
     */
    public function destroy(CompaniesRequest $request, Companies $companies)
    {
        try {
            $companies->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('company::companies.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('company/companies/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('company/companies/' . $companies->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
