<?php

namespace Bixo\Company\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Company\Http\Requests\CompaniesRequest;
use Bixo\Company\Interfaces\CompaniesRepositoryInterface;
use Bixo\Company\Models\Companies;

/**
 * Resource controller class for companies.
 */
class CompaniesResourceController extends BaseController
{

    /**
     * Initialize companies resource controller.
     *
     * @param type CompaniesRepositoryInterface $companies
     *
     * @return null
     */
    public function __construct(CompaniesRepositoryInterface $companies)
    {
        parent::__construct();
        $this->repository = $companies;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Company\Repositories\Criteria\CompaniesResourceCriteria::class);
    }

    /**
     * Display a list of companies.
     *
     * @return Response
     */
    public function index(CompaniesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Company\Repositories\Presenter\CompaniesPresenter::class)
                ->$function();
        }

        $companies = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company::companies.names'))
            ->view('company::companies.index', true)
            ->data(compact('companies', 'view'))
            ->output();
    }

    /**
     * Display companies.
     *
     * @param Request $request
     * @param Model   $companies
     *
     * @return Response
     */
    public function show(CompaniesRequest $request, Companies $companies)
    {

        if ($companies->exists) {
            $view = 'company::companies.show';
        } else {
            $view = 'company::companies.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company::companies.name'))
            ->data(compact('companies'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new companies.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CompaniesRequest $request)
    {

        $companies = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::companies.name')) 
            ->view('company::companies.create', true) 
            ->data(compact('companies'))
            ->output();
    }

    /**
     * Create new companies.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CompaniesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $companies                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::companies.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/companies/' . $companies->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/companies'))
                ->redirect();
        }

    }

    /**
     * Show companies for editing.
     *
     * @param Request $request
     * @param Model   $companies
     *
     * @return Response
     */
    public function edit(CompaniesRequest $request, Companies $companies)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::companies.name'))
            ->view('company::companies.edit', true)
            ->data(compact('companies'))
            ->output();
    }

    /**
     * Update the companies.
     *
     * @param Request $request
     * @param Model   $companies
     *
     * @return Response
     */
    public function update(CompaniesRequest $request, Companies $companies)
    {
        try {
            $attributes = $request->all();

            $companies->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::companies.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/companies/' . $companies->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/companies/' . $companies->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the companies.
     *
     * @param Model   $companies
     *
     * @return Response
     */
    public function destroy(CompaniesRequest $request, Companies $companies)
    {
        try {

            $companies->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::companies.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company/companies/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/companies/' . $companies->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple companies.
     *
     * @param Model   $companies
     *
     * @return Response
     */
    public function delete(CompaniesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::companies.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company/companies'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/companies'))
                ->redirect();
        }

    }

    /**
     * Restore deleted companies.
     *
     * @param Model   $companies
     *
     * @return Response
     */
    public function restore(CompaniesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company::companies.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company/companies'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/companies/'))
                ->redirect();
        }

    }

}
