<?php

namespace Bixo\Company\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Company\Http\Requests\DepartmentsRequest;
use Bixo\Company\Interfaces\DepartmentsRepositoryInterface;
use Bixo\Company\Models\Departments;

/**
 * Resource controller class for departments.
 */
class DepartmentsResourceController extends BaseController
{

    /**
     * Initialize departments resource controller.
     *
     * @param type DepartmentsRepositoryInterface $departments
     *
     * @return null
     */
    public function __construct(DepartmentsRepositoryInterface $departments)
    {
        parent::__construct();
        $this->repository = $departments;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Company\Repositories\Criteria\DepartmentsResourceCriteria::class);
    }

    /**
     * Display a list of departments.
     *
     * @return Response
     */
    public function index(DepartmentsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Company\Repositories\Presenter\DepartmentsPresenter::class)
                ->$function();
        }

        $departments = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company::departments.names'))
            ->view('company::departments.index', true)
            ->data(compact('departments', 'view'))
            ->output();
    }

    /**
     * Display departments.
     *
     * @param Request $request
     * @param Model   $departments
     *
     * @return Response
     */
    public function show(DepartmentsRequest $request, Departments $departments)
    {

        if ($departments->exists) {
            $view = 'company::departments.show';
        } else {
            $view = 'company::departments.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company::departments.name'))
            ->data(compact('departments'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new departments.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DepartmentsRequest $request)
    {

        $departments = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::departments.name')) 
            ->view('company::departments.create', true) 
            ->data(compact('departments'))
            ->output();
    }

    /**
     * Create new departments.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DepartmentsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $departments                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::departments.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/departments/' . $departments->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/departments'))
                ->redirect();
        }

    }

    /**
     * Show departments for editing.
     *
     * @param Request $request
     * @param Model   $departments
     *
     * @return Response
     */
    public function edit(DepartmentsRequest $request, Departments $departments)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::departments.name'))
            ->view('company::departments.edit', true)
            ->data(compact('departments'))
            ->output();
    }

    /**
     * Update the departments.
     *
     * @param Request $request
     * @param Model   $departments
     *
     * @return Response
     */
    public function update(DepartmentsRequest $request, Departments $departments)
    {
        try {
            $attributes = $request->all();

            $departments->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::departments.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/departments/' . $departments->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/departments/' . $departments->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the departments.
     *
     * @param Model   $departments
     *
     * @return Response
     */
    public function destroy(DepartmentsRequest $request, Departments $departments)
    {
        try {

            $departments->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::departments.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company/departments/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/departments/' . $departments->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple departments.
     *
     * @param Model   $departments
     *
     * @return Response
     */
    public function delete(DepartmentsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::departments.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company/departments'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/departments'))
                ->redirect();
        }

    }

    /**
     * Restore deleted departments.
     *
     * @param Model   $departments
     *
     * @return Response
     */
    public function restore(DepartmentsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company::departments.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company/departments'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/departments/'))
                ->redirect();
        }

    }

}
