<?php

namespace Bixo\Company\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Company\Http\Requests\SubdivisionsRequest;
use Bixo\Company\Interfaces\SubdivisionsRepositoryInterface;
use Bixo\Company\Models\Subdivisions;

/**
 * Resource controller class for subdivisions.
 */
class SubdivisionsResourceController extends BaseController
{

    /**
     * Initialize subdivisions resource controller.
     *
     * @param type SubdivisionsRepositoryInterface $subdivisions
     *
     * @return null
     */
    public function __construct(SubdivisionsRepositoryInterface $subdivisions)
    {
        parent::__construct();
        $this->repository = $subdivisions;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Company\Repositories\Criteria\SubdivisionsResourceCriteria::class);
    }

    /**
     * Display a list of subdivisions.
     *
     * @return Response
     */
    public function index(SubdivisionsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Company\Repositories\Presenter\SubdivisionsPresenter::class)
                ->$function();
        }

        $subdivisions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company::subdivisions.names'))
            ->view('company::subdivisions.index', true)
            ->data(compact('subdivisions', 'view'))
            ->output();
    }

    /**
     * Display subdivisions.
     *
     * @param Request $request
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function show(SubdivisionsRequest $request, Subdivisions $subdivisions)
    {

        if ($subdivisions->exists) {
            $view = 'company::subdivisions.show';
        } else {
            $view = 'company::subdivisions.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company::subdivisions.name'))
            ->data(compact('subdivisions'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subdivisions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubdivisionsRequest $request)
    {

        $subdivisions = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::subdivisions.name')) 
            ->view('company::subdivisions.create', true) 
            ->data(compact('subdivisions'))
            ->output();
    }

    /**
     * Create new subdivisions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubdivisionsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subdivisions                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::subdivisions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/subdivisions/' . $subdivisions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/subdivisions'))
                ->redirect();
        }

    }

    /**
     * Show subdivisions for editing.
     *
     * @param Request $request
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function edit(SubdivisionsRequest $request, Subdivisions $subdivisions)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::subdivisions.name'))
            ->view('company::subdivisions.edit', true)
            ->data(compact('subdivisions'))
            ->output();
    }

    /**
     * Update the subdivisions.
     *
     * @param Request $request
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function update(SubdivisionsRequest $request, Subdivisions $subdivisions)
    {
        try {
            $attributes = $request->all();

            $subdivisions->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::subdivisions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/subdivisions/' . $subdivisions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/subdivisions/' . $subdivisions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subdivisions.
     *
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function destroy(SubdivisionsRequest $request, Subdivisions $subdivisions)
    {
        try {

            $subdivisions->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::subdivisions.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company/subdivisions/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/subdivisions/' . $subdivisions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subdivisions.
     *
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function delete(SubdivisionsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::subdivisions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company/subdivisions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/subdivisions'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subdivisions.
     *
     * @param Model   $subdivisions
     *
     * @return Response
     */
    public function restore(SubdivisionsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company::subdivisions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company/subdivisions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/subdivisions/'))
                ->redirect();
        }

    }

}
